const path = require('path')
const yaml = require('js-yaml')
const fs = require('fs')

/**
 *
 */
module.exports = function(customConfig) {
  const {appDir, nodeModulesDir} = customConfig

  const webpackDir = path.join(appDir, 'webpack')
  const webConfig = yaml.load(fs.readFileSync(`${appDir}/config/web.yml`, 'utf8'))
  const webpackConfig = webConfig.webpack

  return {
    context: webpackDir,
    entry: createEntryMap(webpackDir),
    output: {
      path         : '/tmp/perch-app',
      publicPath   : '/',
      filename     : 'js/[name].js',
      chunkFilename: 'js/chunk/[id].js',
    },
    resolve: {
      modules: [
        webpackDir,
        nodeModulesDir,
      ],
      descriptionFiles: ['package.json'],
      mainFields: ['main', 'browser'],
      mainFiles: ['index'],
      extensions: [
        '.js',
      ],
      enforceExtension: false,
      enforceModuleExtension: false,
    },
    plugins: [
    ],
    devServer: {
      contentBase: webpackDir,
      host: '0.0.0.0',
      port: 8081,
      disableHostCheck: true,
      stats: {
        assets: false,
        colors: true,
        children: false,
        chunks: false,
        modules: false,
      },
    },
  }
}

/**
 *
 */
function createEntryMap(webpackDir) {
  const entryMap = {}
  const entriesDir = `${webpackDir}/entries`

  if (fs.existsSync(entriesDir)) {
    for (const entry of getNamesFromDirectory(entriesDir)) {
      entryMap[`entries/${entry}`] = `./entries/${entry}`
    }
  }

  return entryMap
}

/**
 *
 */
function getNamesFromDirectory(filepath) {
  return fs
    .readdirSync(filepath)
    .filter(filename => filename[0] !== '#')
    .map(filename => {
      let entryName, ext
      if (fs.lstatSync(`${filepath}/${filename}`).isDirectory()) {
        entryName = filename
      } else {
        ext = path.extname(filename)
        entryName = filename.substr(0, filename.length - ext.length)
      }
      return entryName
    })
}